@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006-2007 Red Hat, Inc.
+ * Copyright (c) 2006 Red Hat, Inc.
  *
  * All rights reserved.
  *
@@ -175,6 +175,9 @@ static int read_config_file_into_objdb(
 
 	fclose(fp);
 
+	sprintf (error_reason, "Successfully read main configuration file '%s'.\n", filename);
+	*error_string = error_reason;
+
 	return res;
 }
 
