@@ -1,7 +1,7 @@
 /** @file exec/amf.c
  * 
  * Copyright (c) 2002-2006 MontaVista Software, Inc.
- * Author: Steven Dake (sdake@mvista.com)
+ * Author: Steven Dake (sdake@redhat.com)
  *
  * Copyright (c) 2006 Ericsson AB.
  * Author: Hans Feldt, Anders Eriksson, Lars Holm
@@ -150,6 +150,8 @@
 #include "util.h"
 #include "amf.h"
 #include "main.h"
+#include "tlist.h"
+#include "flow.h"
 #include "ipc.h"
 #include "service.h"
 #include "objdb.h"
