@@ -6,7 +6,7 @@
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@mvista.com)
+ * Author: Steven Dake (sdake@redhat.com)
  *
  * Author: Hans Feldt, Anders Eriksson, Lars Holm
  * - Introduced AMF B.02 information model
@@ -140,7 +140,10 @@
 #include "../include/ipc_amf.h"
 #include "totempg.h"
 #include "timer.h"
+#include "flow.h"
+#include "tlist.h"
 #include "ipc.h"
+#include "objdb.h"
 #include "service.h"
 #include "util.h"
 #include "amf.h"
