@@ -194,9 +194,11 @@
 #include <unistd.h>
 #include "amf.h"
 #include "util.h"
-#include "print.h"
+#include "logsys.h"
 #include "main.h"
 
+LOGSYS_DECLARE_SUBSYS ("AMF", LOG_INFO)
+
 /******************************************************************************
  * Internal (static) utility functions
  *****************************************************************************/
@@ -668,12 +670,6 @@ void amf_node_sg_failed_over (struct amf_node *node, struct amf_sg *sg_in)
  * General methods
  *****************************************************************************/
 
-void amf_node_init (void)
-{
-	log_init ("AMF");
-}
-
-
 /**
  * Node constructor
  * @param cluster
