@@ -70,8 +70,11 @@
 #include "../lcr/lcr_comp.h"
 #include "totempg.h"
 #include "main.h"
+#include "flow.h"
+#include "tlist.h"
 #include "ipc.h"
 #include "mempool.h"
+#include "objdb.h"
 #include "service.h"
 #include "logsys.h"
 
@@ -139,6 +142,8 @@ static void clm_sync_abort (void);
 
 static int clm_exec_init_fn (struct objdb_iface_ver0 *objdb);
 
+static int clm_exec_exit_fn (struct objdb_iface_ver0 *objdb);
+
 static int clm_lib_init_fn (void *conn);
 
 static int clm_lib_exit_fn (void *conn);
@@ -213,6 +218,7 @@ struct openais_service_handler clm_service_handler = {
 	.lib_service		= clm_lib_service,
 	.lib_service_count	= sizeof (clm_lib_service) / sizeof (struct openais_lib_handler),
 	.exec_init_fn		= clm_exec_init_fn,
+	.exec_exit_fn		= clm_exec_exit_fn,
 	.exec_dump_fn		= NULL,
 	.exec_service		= clm_exec_service,
 	.exec_service_count	= sizeof (clm_exec_service) / sizeof (struct openais_exec_handler),
@@ -352,6 +358,11 @@ static int clm_exec_init_fn (struct objdb_iface_ver0 *objdb)
 	return (0);
 }
 
+static int clm_exec_exit_fn (struct objdb_iface_ver0 *objdb)
+{
+	return (0);
+}
+
 static int clm_lib_exit_fn (void *conn)
 {
 	struct clm_pd *clm_pd = (struct clm_pd *)openais_conn_private_data_get (conn);
