@@ -59,8 +59,11 @@
 #include "../lcr/lcr_comp.h"
 #include "totempg.h"
 #include "main.h"
+#include "tlist.h"
+#include "flow.h"
 #include "ipc.h"
 #include "mempool.h"
+#include "objdb.h"
 #include "service.h"
 #include "logsys.h"
 
