@@ -5,7 +5,7 @@
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@mvista.com)
+ * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
  *
@@ -49,6 +49,8 @@
 #include <arpa/inet.h>
 
 #include "swab.h"
+#include "objdb.h"
+#include "totem.h"
 #include "service.h"
 #include "../include/saAis.h"
 #include "../include/saLck.h"
@@ -59,6 +61,8 @@
 #include "mempool.h"
 #include "util.h"
 #include "main.h"
+#include "flow.h"
+#include "tlist.h"
 #include "ipc.h"
 #include "totempg.h"
 #include "logsys.h"
@@ -731,7 +735,7 @@ static void message_handler_req_exec_lck_resourceopen (
 	struct resource *resource;
 	struct resource_cleanup *resource_cleanup;
 	SaAisErrorT error = SA_AIS_OK;
-	struct lck_pd *lck_pd = (struct lck_pd *)openais_conn_private_data_get (req_exec_lck_resourceopen->source.conn);
+	struct lck_pd *lck_pd;
 
 	log_printf (LOG_LEVEL_NOTICE, "EXEC request: saLckResourceOpen %s\n",
 		get_mar_name_t (&req_exec_lck_resourceopen->resource_name));
@@ -780,7 +784,8 @@ static void message_handler_req_exec_lck_resourceopen (
 		if (resource_cleanup == 0) {
 			free (resource);
 			error = SA_AIS_ERR_NO_MEMORY;
-		} else {
+		} else { 
+			lck_pd = (struct lck_pd *)openais_conn_private_data_get (req_exec_lck_resourceopen->source.conn);
 			list_init (&resource_cleanup->list);
 			list_init (&resource_cleanup->resource_lock_list_head);
 			resource_cleanup->resource = resource;
@@ -1016,9 +1021,7 @@ void lock_algorithm (
 		/*
 		 * Exclusive lock granted
 		 */
-		if (resource_lock->lock_mode == SA_LCK_PR_LOCK_MODE) {
-			lock_queue (resource, resource_lock);
-		}
+		lock_queue (resource, resource_lock);
 	} else {
 		/*
 		 * Exclusive lock not granted
