@@ -5,7 +5,7 @@
  *
  * All rights reserved.
  *
- * Author: Steven Dake (sdake@mvista.com)
+ * Author: Steven Dake (sdake@redhat.com)
  *
  * This software licensed under BSD license, the text of which follows:
  *
@@ -73,6 +73,8 @@
 #include "swab.h"
 #include "objdb.h"
 #include "config.h"
+#include "tlist.h"
+#include "flow.h"
 #include "ipc.h"
 #include "timer.h"
 #include "logsys.h"
@@ -101,6 +103,12 @@ static struct totem_logging_configuration totem_logging_configuration;
 
 static char delivery_data[MESSAGE_SIZE_MAX];
 
+static int num_config_modules;
+
+static struct config_iface_ver0 *config_modules[MAX_DYNAMIC_SERVICES];
+
+static struct objdb_iface_ver0 *objdb = NULL;
+
 SaClmClusterNodeT *(*main_clm_get_by_nodeid) (unsigned int node_id);
 
 static void sigusr2_handler (int num)
@@ -114,6 +122,50 @@ static void sigusr2_handler (int num)
 	}
 }
 
+static void *aisexec_exit (void *arg)
+{
+	if(objdb) {
+		openais_service_unlink_all (objdb);
+	}
+
+#ifdef DEBUG_MEMPOOL
+	int stats_inuse[MEMPOOL_GROUP_SIZE];
+	int stats_avail[MEMPOOL_GROUP_SIZE];
+	int stats_memoryused[MEMPOOL_GROUP_SIZE];
+	int i;
+
+	mempool_getstats (stats_inuse, stats_avail, stats_memoryused);
+	log_printf (LOG_LEVEL_DEBUG, "Memory pools:\n");
+	for (i = 0; i < MEMPOOL_GROUP_SIZE; i++) {
+	log_printf (LOG_LEVEL_DEBUG, "order %d size %d inuse %d avail %d memory used %d\n",
+		i, 1<<i, stats_inuse[i], stats_avail[i], stats_memoryused[i]);
+	}
+#endif
+
+	totempg_finalize ();
+	logsys_flush ();
+
+	openais_exit_error (AIS_DONE_EXIT);
+
+	/* never reached */
+	return NULL;
+}
+
+pthread_t aisexec_exit_thread;
+static void init_shutdown(void *data) 
+{
+	pthread_create (&aisexec_exit_thread, NULL, aisexec_exit, NULL);
+}
+
+
+static poll_timer_handle shutdown_handle;
+static void sigquit_handler (int num)
+{
+	/* avoid creating threads from within the interrupt context */
+	poll_timer_add (aisexec_poll_handle, 500, NULL, init_shutdown, &shutdown_handle);
+}
+
+
 static void sigsegv_handler (int num)
 {
 	signal (SIGSEGV, SIG_DFL);
@@ -139,23 +191,7 @@ struct totempg_group openais_group = {
 
 void sigintr_handler (int signum)
 {
-
-#ifdef DEBUG_MEMPOOL
-	int stats_inuse[MEMPOOL_GROUP_SIZE];
-	int stats_avail[MEMPOOL_GROUP_SIZE];
-	int stats_memoryused[MEMPOOL_GROUP_SIZE];
-	int i;
-
-	mempool_getstats (stats_inuse, stats_avail, stats_memoryused);
-	log_printf (LOG_LEVEL_DEBUG, "Memory pools:\n");
-	for (i = 0; i < MEMPOOL_GROUP_SIZE; i++) {
-	log_printf (LOG_LEVEL_DEBUG, "order %d size %d inuse %d avail %d memory used %d\n",
-		i, 1<<i, stats_inuse[i], stats_avail[i], stats_memoryused[i]);
-	}
-#endif
-
-	totempg_finalize ();
-	openais_exit_error (AIS_DONE_EXIT);
+	poll_timer_add (aisexec_poll_handle, 500, NULL, init_shutdown, &shutdown_handle);
 }
 
 
@@ -278,9 +314,13 @@ static void aisexec_mempool_init (void)
 
 static void aisexec_tty_detach (void)
 {
+	int lpc, fd;
+	struct rlimit oflimits;
+
 	/*
 	 * Disconnect from TTY if this is not a debug run
 	 */
+
 	switch (fork ()) {
 		case -1:
 			openais_exit_error (AIS_DONE_FORK);
@@ -289,11 +329,33 @@ static void aisexec_tty_detach (void)
 			/*
 			 * child which is disconnected, run this process
 			 */
+/* 			setset(); */
+			close (0);
+			close (1);
+			close (2);
 			break;
 		default:
 			exit (0);
 			break;
 	}
+
+	/* Create new session */
+	setsid();
+
+	/* 
+	 * Map stdin/out/err to /dev/null.
+	 */
+	fd = open("/dev/null", O_RDWR);
+	if (fd >= 0) {
+		/* dup2 to 0 / 1 / 2 (stdin / stdout / stderr) */
+		dup2(fd, STDIN_FILENO);  /* 0 */
+		dup2(fd, STDOUT_FILENO); /* 1 */
+		dup2(fd, STDERR_FILENO); /* 2 */
+
+		/* Should be 0, but just in case it isn't... */
+		if (fd > 2) 
+			close(fd);
+	}
 }
 
 static void aisexec_setscheduler (void)
@@ -395,6 +457,12 @@ static void deliver_fn (
 		(header, nodeid);
 }
 
+void main_get_config_modules(struct config_iface_ver0 ***modules, int *num)
+{
+	*modules = config_modules;
+	*num = num_config_modules;
+}
+
 int main (int argc, char **argv)
 {
 	char *error_string;
@@ -403,11 +471,11 @@ int main (int argc, char **argv)
 	unsigned int objdb_handle;
 	unsigned int config_handle;
 	unsigned int config_version = 0;
-	struct objdb_iface_ver0 *objdb;
 	void *objdb_p;
 	struct config_iface_ver0 *config;
 	void *config_p;
 	char *config_iface;
+	char *iface;
 	int res, ch;
 	int background, setprio;
  	int totem_log_service;
@@ -422,6 +490,7 @@ int main (int argc, char **argv)
 		switch (ch) {
 			case 'f':
 				background = 0;
+				logsys_config_mode_set (LOG_MODE_OUTPUT_STDERR|LOG_MODE_FLUSH_AFTER_CONFIG);
 				break;
 			case 'p':
 				setprio = 0;
@@ -446,7 +515,8 @@ int main (int argc, char **argv)
 	signal (SIGUSR2, sigusr2_handler);
 	signal (SIGSEGV, sigsegv_handler);
 	signal (SIGABRT, sigabrt_handler);
-
+	signal (SIGQUIT, sigquit_handler);
+	
 	openais_timer_init (
 		serialize_mutex_lock,
 		serialize_mutex_unlock);
@@ -475,39 +545,49 @@ int main (int argc, char **argv)
 
 	objdb->objdb_init ();
 
-	/* User's bootstrap config service */
+	num_config_modules = 0;
+
+	/*
+	 * Bootstrap in the default configuration parser or use
+	 * the openais default built in parser if the configuration parser
+	 * isn't overridden
+	 */
 	config_iface = getenv("OPENAIS_DEFAULT_CONFIG_IFACE");
 	if (!config_iface) {
 		config_iface = "aisparser";
 	}
 
-	res = lcr_ifact_reference (
-		&config_handle,
-		config_iface,
-		config_version,
-		&config_p,
-		0);
+	/* Make a copy so we can deface it with strtok */
+	config_iface = strdup(config_iface);
 
-	config = (struct config_iface_ver0 *)config_p;
-	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, "AIS Executive couldn't open configuration component.\n");
-		openais_exit_error (AIS_DONE_MAINCONFIGREAD);
-	}
+	iface = strtok(config_iface, ":");
+	while (iface)
+	{
+		res = lcr_ifact_reference (
+			&config_handle,
+			iface,
+			config_version,
+			&config_p,
+			0);
 
-	res = config->config_readconfig(objdb, &error_string);
-	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, error_string);
-		openais_exit_error (AIS_DONE_MAINCONFIGREAD);
-	}
-	log_printf (LOG_LEVEL_NOTICE, error_string);
+		config = (struct config_iface_ver0 *)config_p;
+		if (res == -1) {
+			log_printf (LOG_LEVEL_ERROR, "AIS Executive couldn't open configuration component '%s'\n", iface);
+			openais_exit_error (AIS_DONE_MAINCONFIGREAD);
+		}
 
-	openais_service_default_objdb_set (objdb);
+		res = config->config_readconfig(objdb, &error_string);
+		if (res == -1) {
+			log_printf (LOG_LEVEL_ERROR, error_string);
+			openais_exit_error (AIS_DONE_MAINCONFIGREAD);
+		}
+		log_printf (LOG_LEVEL_NOTICE, error_string);
+		config_modules[num_config_modules++] = config;
 
-	res = openais_service_link_all (objdb);
-	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, "Could not load services\n");
-		openais_exit_error (AIS_DONE_DYNAMICLOAD);
+		iface = strtok(NULL, ":");
 	}
+	if (config_iface)
+		free(config_iface);
 
 	res = openais_main_config_read (objdb, &error_string, &main_config);
 	if (res == -1) {
@@ -533,13 +613,14 @@ int main (int argc, char **argv)
 		openais_exit_error (AIS_DONE_MAINCONFIGREAD);
 	}
 
+	logsys_config_facility_set ("openais", main_config.syslog_facility);
+	logsys_config_mode_set (main_config.logmode);
+
 	res = logsys_config_file_set (&error_string, main_config.logfile);
 	if (res == -1) {
 		log_printf (LOG_LEVEL_ERROR, error_string);
 		openais_exit_error (AIS_DONE_LOGSETUP);
 	}
-	logsys_config_facility_set ("openais", main_config.syslog_facility);
-	logsys_config_mode_set (main_config.logmode);
 
 	aisexec_uid_determine (&main_config);
 
@@ -599,12 +680,13 @@ int main (int argc, char **argv)
 	/*
 	 * This must occur after totempg is initialized because "this_ip" must be set
 	 */
-	res = openais_service_init_all (service_count, objdb);
+	res = openais_service_defaults_link_and_init (objdb);
 	if (res == -1) {
-		log_printf (LOG_LEVEL_ERROR, "Could not init services\n");
+		log_printf (LOG_LEVEL_ERROR, "Could not initialize default services\n");
 		openais_exit_error (AIS_DONE_INIT_SERVICES);
 	}
 
+
 	sync_register (openais_sync_callbacks_retrieve, openais_sync_completed,
 		totem_config.vsf_type);
 
