@@ -45,11 +45,11 @@
 #include <sys/socket.h>
 #include <errno.h>
 
-#include "../include/saAis.h"
-#include "../include/cpg.h"
-#include "../include/ipc_cpg.h"
-#include "../include/mar_cpg.h"
-#include "util.h"
+#include <saAis.h>
+#include <cpg.h>
+#include <ipc_cpg.h>
+#include <mar_cpg.h>
+#include <ais_util.h>
 
 struct cpg_inst {
 	int response_fd;
@@ -57,6 +57,7 @@ struct cpg_inst {
 	int finalize;
 	cpg_flow_control_state_t flow_control_state;
 	cpg_callbacks_t callbacks;
+	void *context;
 	pthread_mutex_t response_mutex;
 	pthread_mutex_t dispatch_mutex;
 };
@@ -75,6 +76,10 @@ static struct saHandleDatabase cpg_handle_t_db = {
  */
 static void cpg_instance_destructor (void *instance)
 {
+	struct cpg_inst *cpg_inst = instance;
+
+	pthread_mutex_destroy (&cpg_inst->response_mutex);
+	pthread_mutex_destroy (&cpg_inst->dispatch_mutex);
 }
 
 
@@ -190,6 +195,44 @@ cpg_error_t cpg_fd_get (
 	return (SA_AIS_OK);
 }
 
+cpg_error_t cpg_context_get (
+	cpg_handle_t handle,
+	void **context)
+{
+	SaAisErrorT error;
+	struct cpg_inst *cpg_inst;
+
+	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	if (error != SA_AIS_OK) {
+		return (error);
+	}
+
+	*context = cpg_inst->context;
+
+	saHandleInstancePut (&cpg_handle_t_db, handle);
+
+	return (SA_AIS_OK);
+}
+
+cpg_error_t cpg_context_set (
+	cpg_handle_t handle,
+	void *context)
+{
+	SaAisErrorT error;
+	struct cpg_inst *cpg_inst;
+
+	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	if (error != SA_AIS_OK) {
+		return (error);
+	}
+
+	cpg_inst->context = context;
+
+	saHandleInstancePut (&cpg_handle_t_db, handle);
+
+	return (SA_AIS_OK);
+}
+
 struct res_overlay {
 	mar_res_header_t header __attribute__((aligned(8)));
 	char data[512000];
@@ -605,6 +648,48 @@ error_exit:
 	return (error);
 }
 
+cpg_error_t cpg_local_get (
+	cpg_handle_t handle,
+	unsigned int *local_nodeid)
+{
+	cpg_error_t error;
+	struct cpg_inst *cpg_inst;
+	struct iovec iov;
+	struct req_lib_cpg_local_get req_lib_cpg_local_get;
+	struct res_lib_cpg_local_get res_lib_cpg_local_get;
+
+	error = saHandleInstanceGet (&cpg_handle_t_db, handle, (void *)&cpg_inst);
+	if (error != SA_AIS_OK) {
+		return (error);
+	}
+
+	req_lib_cpg_local_get.header.size = sizeof (mar_req_header_t);
+	req_lib_cpg_local_get.header.id = MESSAGE_REQ_CPG_LOCAL_GET;
+
+	iov.iov_base = &req_lib_cpg_local_get;
+	iov.iov_len = sizeof (struct req_lib_cpg_local_get);
+
+	pthread_mutex_lock (&cpg_inst->response_mutex);
+
+	error = saSendMsgReceiveReply (cpg_inst->response_fd, &iov, 1,
+		&res_lib_cpg_local_get, sizeof (res_lib_cpg_local_get));
+
+	pthread_mutex_unlock (&cpg_inst->response_mutex);
+
+	if (error != SA_AIS_OK) {
+		goto error_exit;
+	}
+
+	error = res_lib_cpg_local_get.header.error;
+
+	*local_nodeid = res_lib_cpg_local_get.local_nodeid;
+
+error_exit:
+	saHandleInstancePut (&cpg_handle_t_db, handle);
+
+	return (error);
+}
+
 cpg_error_t cpg_flow_control_state_get (
 	cpg_handle_t handle,
 	cpg_flow_control_state_t *flow_control_state)
